#!/usr/bin/python
# -*- encoding: utf-8 -*-

# Priklad ovladani obvodu PCA9685: 
#      Nastaveni vystupu PWM0 a PWM1 na frekvenci 100Hz se stridou 10%
#      PWM1 je za PWM0 zpozdene o ctvrtinu periody

import time
import smbus


bus = smbus.SMBus(1)   # novejsi varianta RasPi (512MB)
#bus = smbus.SMBus(0)   # starsi varianta RasPi (256MB)

addr = 0x40            # I2C adresa obvodu PCA9685 (kdyz jsou vsechny adresovaci vstupy v "0", je adresa 0x40)


# kanal "A" se nastavuje stejne, jako v prvnim prikladu
gen=25000000                                   # frekvence vnitrniho generatoru obvodu PCA9685 (25MHz)
frekvence = 30                                 # pozadovana frekvence vystupu v Hz (23.84 az 6103)

strida_A = 0.1                                 # sirka pulzu signalu "A" 0.1 = 10%
kanal_A = 2                                    # cislo kanalu "A" 0 az 15

strida_B = 0.1                                 # sirka pulzu signalu "B" 0.1 = 10%
kanal_B = 3                                    # cislo kanalu "B" 0 az 15
zpozdeni_B = 0.25                              # zpozdeni startu signalu "B" proti signalu "A" 0.25 = 25% periody

n = int((gen /frekvence) / 4096)-1             # prepocet frekvence na pozadovanou konstantu prescaleru (orezava)
nastavena_frekv = gen / (4096.0 * (n+1))       # zpetny vypocet skutecne nastavene frekvence

sirka_H_A = int(4095 * strida_A ) & 0xFFF      # sirka impulzu se nasatvuje v rozsahu 0 az 4095 pro 0% az 100%

adr_kan_A = 6 + (kanal_A * 4)                  # kazdy kanal se ridi pomomci 4 bajtu. Nulty kanal zacina na adrese 6

dolnibajt_H_A = int(sirka_H_A % 256)           # prepocet 12-bitove sirky impulzu "A" na horni a dolni bajt
hornibajt_H_A = int(sirka_H_A / 256)



# pro kanal "B" se musi navic vypocitat zpozdeni startu

sirka_L_B = int(4095 * zpozdeni_B ) & 0xFFF    # sirka nuly se nasatvuje v rozsahu 0 az 4095 pro 0% az 100% zpozdeni

sirka_H_B = int(4095 * strida_A ) + sirka_L_B & 0xFFF   # sirka impulzu se nasatvuje v rozsahu 0 az 4095 pro 0% az 100%

adr_kan_B = 6 + (kanal_B * 4)                  # kazdy kanal se ridi pomomci 4 bajtu. Nulty kanal zacina na adrese 6

dolnibajt_L_B = int(sirka_L_B % 256)           # prepocet 12-bitove sirky pocatecni nuly na horni a dolni bajt
hornibajt_L_B = int(sirka_L_B / 256)
dolnibajt_H_B = int(sirka_H_B % 256)           # prepocet 12-bitove sirky impulzu na horni a dolni bajt
hornibajt_H_B = int(sirka_H_B / 256)


print "Skutecne nastavena frekvence:  " + str(nastavena_frekv)  + " (prescaler (0 az 255): " + str(n) + ")"
print "Skutecna sirka signalu A (0 az 4095):  " + str(sirka_H_A) + "   (" + str(sirka_H_A / 40.95) + "%)"
print "Skutecna sirka signalu B (0 az 4095):  " + str(sirka_H_B - sirka_L_B) + "   (" + str((sirka_H_B - sirka_L_B) / 40.95) + "%)"
print "Signal B jde z '0' do '1' v case (0 az 4095):  " + str(sirka_L_B) + "   (" + str(sirka_L_B / 40.95) + "%)"
print "Signal B jde z '1' do '0' v case (0 az 4095):  " + str(sirka_H_B) + "   (" + str(sirka_H_B / 40.95) + "%)"



# inicializace PWM s pouzitim vnitrniho oscilatoru (25MHz)
bus.write_byte_data(addr,0x00,0b00010000)  # SLEEP
bus.write_byte_data(addr,0xFE,n)           # nastaveni frekvence prescaleru
bus.write_byte_data(addr,0x01,0b00010000)  # Logika vystupu: Ledka sviti pri "0" (je zapojena proti VCC)


# nastaveni parametru kanalu "A"
bus.write_byte_data(addr, adr_kan_A,    0)     # prechod signalu z "0" na "1" v case "T + nula"      8xLSB
bus.write_byte_data(addr, adr_kan_A + 1,0)     #                                                     4xMSB
bus.write_byte_data(addr, adr_kan_A + 2, dolnibajt_H_A) #  prechod z "1" na "0" v case "sirka_H_A"   8xLSB
bus.write_byte_data(addr, adr_kan_A + 3, hornibajt_H_A) #                                            4xMSB
  
# nastaveni parametru kanalu "B"
bus.write_byte_data(addr, adr_kan_B,     dolnibajt_L_B) # prechod signalu z "0" na "1" v case "zpozdeni_B"        8xLSB
bus.write_byte_data(addr, adr_kan_B + 1, hornibajt_L_B) #                                                         4xMSB
bus.write_byte_data(addr, adr_kan_B + 2, dolnibajt_H_B) #  prechod z "1" na "0" v case "zpozdeni_B" + "sirka_H_B" 8xLSB
bus.write_byte_data(addr, adr_kan_B + 3, hornibajt_H_B) #                                                         4xMSB



  
# spusteni generatoru na 5 sekund    
bus.write_byte_data(addr,0x00,0b00000000)  # spust PWM generator
time.sleep(5)                              # 5 sekund generovani signalu
bus.write_byte_data(addr,0x00,0b00010000)  # zastav PWM generator (SLEEP) (bez zastaveni pobezi neustale nezavisle na RasPi)


